; code handling the challenges

!RAM_challenge_indicator_pos = $79 ; freeram, clears on level load
!RAM_challenge_indicator_direction = $7C ; freeram, clears on level load
!RAM_challenge_indicator_timer = $87 ; freeram, clears on level load

!RAM_challenge_indicator_oam = $60 ; freeram, 2 bytes, set to the OAM tile to display for the challenge
!RAM_challenge_indicator_oam2 = $62 ; freeram, 2 bytes, set to the OAM tile to display for the challenge (above the X, only used if not 0)

!RAM_challenge_message_flag = $0D9C ; flags to keep track of whether or not the challenge message was shown as well as altering the 105-1 message to the challenge text. RAM is only reset on reset/titlescreen
; format m------t, m - load alternate 105-1 message (challenge info text), t - don't display again (bit 7 is reset once that message is shown, non-0 means "don't show again")

!RAM_OW_ACTION = $0DD5
!DEF_OW_ACTION_NORMAL_EXIT = $01
!DEF_OW_ACTION_SECRET_EXIT1 = $02
!DEF_OW_ACTION_SECRET_EXIT2 = $03
!DEF_OW_ACTION_SECRET_EXIT3 = $04

!TL = 0
!TR = 1
!BL = 2
!BR = 3
function get_memory_index(index, screen, quarter) = $19F8+(index*128)+(screen*4)+quarter
function yxppccct(yflip, xflip, priority, palette, second_page) = (yflip<<7)|(xflip<<6)|(priority<<4)|((palette-8)<<1)|(second_page)

!RAM_challenge_failed_flag #= get_memory_index(0, 5, !TR) ; (index 0, screen 5, top right) ; this byte corresponds to the first yoshi block in room 106. Bit 0 will be set when offscreen yoshi tongue has been triggered, that bit is not used anywhere in the level.

; $1F2B is used by dacin.asm
!RAM_challenge_normal_exit = $1F2C ; saved to SRAM
!RAM_challenge_secret_exit_2 = $1F2D ; saved to SRAM

!SRAM_challenge_normal_exit #= !RAM_challenge_normal_exit-$1F2B+$700089
!SRAM_challenge_secret_exit_2 #= !RAM_challenge_secret_exit_2-$1F2B+$700089

!SRAM_exit_count = $70008C
macro oam_jml(addr)
; assumes 16-bit A
	LDA.w #$005C|((<addr><<8)&$FF00) ; 5C = JML
	STA $7F817F
	LDA.w #<addr>>>8
	STA $7F8181
endmacro

org $009329+($0C*2)
	; dw $A087 ; original code
	dw GM0C_hijack
warnpc $009329+($0D*2)

org $00A1C7 ; JSL GameMode_0E_Prim [$048241]
	autoclean JSL GM0E_hijack

org $0091AB
	autoclean JML level_load
level_load_return = $8091B0 ; RTS

org $00F9F5 ; empty area in bank 0
GM0C_hijack:
	LDA.w !RAM_challenge_failed_flag
	BNE .return ; return if we have failed the challenge

	; check if we triggered an exit and set the corresponding challenge beaten flag
	LDA.w !RAM_OW_ACTION
	CMP.b #!DEF_OW_ACTION_NORMAL_EXIT
	BEQ .normal_exit
	CMP.b #!DEF_OW_ACTION_SECRET_EXIT2
	BNE .return
	; LDA #$01 ; only bit 0 needs to be set, which is the case for DEF_OW_ACTION_SECRET_EXIT2
	STA.w !RAM_challenge_secret_exit_2
	BRA .return ; save a byte
	; JMP $A087 ; jump to actual GM14 code

.normal_exit
	; LDA #$01 ; already 01
	STA !RAM_challenge_normal_exit
.return
	JMP $A087 ; jump to actual GM14 code
warnpc $00FA10

org $00C0C1 ; JSR $C00D ; set item memory
	JSR SetItemMemoryHijack ; set item memory for brown block

org $00FF93 ; empty area in bank 0
SetItemMemoryHijack:
	JSR $C00D ; call actual routine
	CPY #$0015	; yoshi block index in room 2
	BNE .no_challenge
	; CPX #$0003	; yoshi block index in room 2 ; skip checking for specific bit, the entire byte can only be written to using a brown block by this yoshi block.
	; BNE .no_challenge
	; challenge failed, the yoshi block in room 2 was triggered -> show indicator
	REP #$20
		%oam_jml(OAM_hijack_set_block)
	SEP #$20
.no_challenge
	RTS
warnpc $00FFC0 ; don't overwrite snes ROM header

org $01F321 ; JSR $01F3FE
	JSR yoshi_tongue_gfx_hijack

org $01FFBF ; empty area in bank 1
yoshi_tongue_gfx_hijack:
	LDA.w $15A0,x				;$01F3FE	|\
	ORA.w $186C,x				;$01F401	|| Return if offscreen or entering pipe.
	ORA.w $1419					;$01F404	||
	BNE .failed 				;$01F407	|/
	JMP $F409 ; return to normal code

.failed
	; If the visible yoshi is using it's tongue while offscreen, then set the failure flag.
	; LDA !RAM_challenge_failed_flag
	; BNE .already_failed
	LDA #$01
	TSB !RAM_challenge_failed_flag
	REP #$20
		LDA.w #yxppccct(0, 0, 3, $0D, 1)<<8|$E0
		STA !RAM_challenge_indicator_oam
		STZ !RAM_challenge_indicator_oam2
		%oam_jml(OAM_hijack) ; and show the yoshi failed indicator
	SEP #$20
.already_failed
	RTS ; restore normal code
warnpc $01FFF0 ; 01FFF0-01FFF8 is used by dacin.asm

!DEF_tile = $2E
!DEF_props = %00110110
macro oam_tile(x, y, tile, props)
	!DEF_OAM_SLOT ?= 0
	LDA $14
	ASL #2
	AND #$40
	ORA.b #<props>
	XBA
	LDA.b #<tile>
	REP #$20
		STA $0302+!DEF_OAM_SLOT
		LDA.w #<y><<8|<x>
		STA $0300+!DEF_OAM_SLOT
	SEP #$20
	LDA #$02
	STA $0460+(!DEF_OAM_SLOT/4)
	!DEF_OAM_SLOT #= !DEF_OAM_SLOT+4
endmacro

freecode
GM0E_hijack:
	LDA !RAM_challenge_normal_exit
	BEQ .no_normal_exit
	%oam_tile(($F0-$30), $38, !DEF_tile, !DEF_props)
.no_normal_exit
	LDA !RAM_challenge_secret_exit_2
	BEQ .no_secret_exit
	%oam_tile($30, $38, !DEF_tile, !DEF_props)
.no_secret_exit
	JML $848241 ; back to original OW code

OAM_hijack_set_block:
	LDA #$2A
	STA !RAM_challenge_indicator_oam
	LDA.b #yxppccct(0, 0, 3, $08, 0)
	STA !RAM_challenge_indicator_oam+1
	LDA #$8E
	STA !RAM_challenge_indicator_oam2
	LDA.b #yxppccct(0, 0, 3, $0D, 1)
	STA !RAM_challenge_indicator_oam2+1
	LDA #$F0
OAM_hijack:
	STA $03FD ; restore overwritten code
	BIT $1F02
	BVC .quitOnly ; quit immediately if we haven't beaten the main exit yet
	LDA $141A
	CMP #$02
	BCS .quitOnly ; quit immediately if we are past room 2, I refuse to believe you can reach there without failing either of the challenge conditions. Please don't prove me wrong.
	!DEF_OAM_SLOT = $B0
	LDA #$E2 ; X tile
	STA $0202+!DEF_OAM_SLOT
	LDA !RAM_challenge_indicator_oam
	STA $0202+!DEF_OAM_SLOT+4
	LDA.b #yxppccct(0, 0, 3, $0C, 1) ; X properties (yflip, xyflip, priority, palette, second_page)
	STA $0203+!DEF_OAM_SLOT
	LDA !RAM_challenge_indicator_oam+1
	STA $0203+!DEF_OAM_SLOT+4
	LDA #$08
	STA $0200+!DEF_OAM_SLOT ; x position
	STA $0200+!DEF_OAM_SLOT+4 ; x position
	LDA #$E0
	SEC : SBC !RAM_challenge_indicator_pos
	STA $0201+!DEF_OAM_SLOT ; y position
	STA $0201+!DEF_OAM_SLOT+4 ; y position
	LDA #$02
	STA $0420+(!DEF_OAM_SLOT/4) ; tile size
	STA $0420+(!DEF_OAM_SLOT/4)+1 ; tile size
	
	LDA !RAM_challenge_indicator_oam2
	BEQ +
	JSR SetIndicator2
+
	; handle moving the tile onscreen, waiting a bit then back offscreen
	LDA $14
	AND #$01
	BNE .end
	LDA !RAM_challenge_indicator_direction
	BNE .go_in_direction
	DEC !RAM_challenge_indicator_timer
	BPL .end
	LDA !RAM_challenge_indicator_pos
	BEQ .increase
	DEC !RAM_challenge_indicator_direction
	BRA .go_in_direction_00
.increase
	INC !RAM_challenge_indicator_direction
	LDA #$80
	STA !RAM_challenge_indicator_timer
.go_in_direction_00
	LDA !RAM_challenge_indicator_direction
.go_in_direction
	CLC : ADC !RAM_challenge_indicator_pos
	BEQ .quit
	CMP #$10
	BNE .store
	STZ !RAM_challenge_indicator_direction
.store
	STA !RAM_challenge_indicator_pos
	BRA .end
.quit
	STZ !RAM_challenge_indicator_pos
	STZ !RAM_challenge_indicator_direction
.quitOnly
	REP #$20
		LDA.w #$FD8D ; \
		STA $7F817F  ; | STA $03FD : RTL
		LDA.w #$6B03 ; /
		STA $7F8181
	SEP #$20

.end
	LDA #$F0 ; just in case some dumb code relies on the value of A after the call
	RTL

SetIndicator2:
	STA $0202+!DEF_OAM_SLOT+8
	LDA !RAM_challenge_indicator_oam2+1
	STA $0203+!DEF_OAM_SLOT+8
	LDA #$08
	STA $0200+!DEF_OAM_SLOT+8 ; x position
	LDA $0201+!DEF_OAM_SLOT+4 ; y position indicator tile
	SEC : SBC #$10
	STA $0201+!DEF_OAM_SLOT+8 ; y position additional tile
	LDA #$02
	STA $0420+(!DEF_OAM_SLOT/4)+2 ; tile size additional tile
	RTS

level_load:
	LDA #$1E
	STA $0DC0 ; restore hijacked code. (set amount of coins needed to get 1up from green star block)
	
	; code runs at the end of GM12 (before level fade-in) if entering a level from the overworld (not a sublevel)
	BIT $1F02
	BVC .rts ; quit immediately if we haven't beaten the main exit yet
	LDA !RAM_challenge_message_flag
	BNE .rts ; return if we have already shown the message
	LDA #$01
	STA $1426 ; show message box 1
	LDA #$81
	STA !RAM_challenge_message_flag ; load an alternate message for 105-1 and mark as displayed so we don't display it again
.rts
	JML .return

; File Select stuff, draw a star after the exit count for every challenge beaten
!DEF_empty_tile = $38FC
!DEF_star_tile = $3887

!TMP_challenges_count = $01

org $009D66
	LDA.l !SRAM_challenge_secret_exit_2,x
	AND #$01
	CLC : ADC.l !SRAM_challenge_normal_exit,x
	STA !TMP_challenges_count
	STZ !TMP_challenges_count+1 ; clear high byte because it's read in 16-bit mode later
	TDC ; 16-bit clear A
	LDA !SRAM_exit_count,x
	REP #$20
	SEP #$10
	ORA #$3800
	LDX $00
	STA.l $7F8381,x ; Draw number of exits (hack only has 3 exits, no need for HexToDec)
	LDA.w #!DEF_star_tile
	LDY #$02
-	DEC !TMP_challenges_count
	BPL +
	LDA.w #!DEF_empty_tile
+	STA $7F8383,x
	INX #2
	DEY
	BPL -

	STA $7F837F-($03*2),x ; clear out tens place. (!TMP_challenges_count can only be 2 max so A will always be set to !DEF_empty_tile at this point)
	SEP #$20
	BRA +
warnpc $009DA6
org $009DA6
+
